@@ -1,4 +1,4 @@
-__version__ = '0.1.5'
+__version__ = '0.2.0'
 
 __author__ = 'Nima Mahmoudi <nma@ieee.org>'
 
@@ -9,3 +9,5 @@ from . import SimProcess
 from . import Utility
 from . import ServerlessSimulator
 from . import FunctionInstance
+from . import ParFunctionInstance
+from . import ParServerlessSimulator
